@@ -76,34 +76,6 @@ static const char *type2string(enum connman_element_type type)
 	return NULL;
 }
 
-const char *__connman_ipv4_method2string(enum connman_ipv4_method method)
-{
-	switch (method) {
-	case CONNMAN_IPV4_METHOD_UNKNOWN:
-		return "unknown";
-	case CONNMAN_IPV4_METHOD_OFF:
-		return "off";
-	case CONNMAN_IPV4_METHOD_STATIC:
-		return "static";
-	case CONNMAN_IPV4_METHOD_DHCP:
-		return "dhcp";
-	}
-
-	return "unknown";
-}
-
-enum connman_ipv4_method __connman_ipv4_string2method(const char *method)
-{
-	if (strcasecmp(method, "off") == 0)
-		return CONNMAN_IPV4_METHOD_OFF;
-	else if (strcasecmp(method, "static") == 0)
-		return CONNMAN_IPV4_METHOD_STATIC;
-	else if (strcasecmp(method, "dhcp") == 0)
-		return CONNMAN_IPV4_METHOD_DHCP;
-	else
-		return CONNMAN_IPV4_METHOD_UNKNOWN;
-}
-
 static void emit_element_signal(DBusConnection *conn, const char *member,
 					struct connman_element *element)
 {
@@ -272,11 +244,62 @@ int __connman_element_count(struct connman_element *element,
 	return data.count;
 }
 
-const char *__connman_element_get_device(struct connman_element *element)
+static struct connman_network *__connman_element_get_network(struct connman_element *element)
+{
+	if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK &&
+						element->network != NULL)
+		return element->network;
+
+	if (element->parent == NULL)
+		return NULL;
+
+	return __connman_element_get_network(element->parent);
+}
+
+struct connman_service *__connman_element_get_service(struct connman_element *element)
+{
+	struct connman_service *service = NULL;
+	struct connman_network *network;
+	struct connman_device *device;
+	enum connman_device_type type;
+
+	device = __connman_element_get_device(element);
+	if (device == NULL)
+		return NULL;
+
+	type = connman_device_get_type(device);
+
+	switch (type) {
+	case CONNMAN_DEVICE_TYPE_UNKNOWN:
+	case CONNMAN_DEVICE_TYPE_VENDOR:
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+	case CONNMAN_DEVICE_TYPE_GPS:
+	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_HSO:
+	case CONNMAN_DEVICE_TYPE_NOZOMI:
+	case CONNMAN_DEVICE_TYPE_HUAWEI:
+	case CONNMAN_DEVICE_TYPE_NOVATEL:
+		break;
+	case CONNMAN_DEVICE_TYPE_ETHERNET:
+		service = __connman_service_lookup_from_device(device);
+		break;
+	case CONNMAN_DEVICE_TYPE_WIFI:
+	case CONNMAN_DEVICE_TYPE_WIMAX:
+		network = __connman_element_get_network(element);
+		if (network == NULL)
+			return NULL;
+		service = __connman_service_lookup_from_network(network);
+		break;
+	}
+
+	return service;
+}
+
+struct connman_device *__connman_element_get_device(struct connman_element *element)
 {
 	if (element->type == CONNMAN_ELEMENT_TYPE_DEVICE &&
 						element->device != NULL)
-		return element->path;
+		return element->device;
 
 	if (element->parent == NULL)
 		return NULL;
@@ -284,7 +307,18 @@ const char *__connman_element_get_device(struct connman_element *element)
 	return __connman_element_get_device(element->parent);
 }
 
-const char *__connman_element_get_network(struct connman_element *element)
+const char *__connman_element_get_device_path(struct connman_element *element)
+{
+	struct connman_device *device;
+
+	device = __connman_element_get_device(element);
+	if (device == NULL)
+		return NULL;
+
+	return connman_device_get_path(device);
+}
+
+const char *__connman_element_get_network_path(struct connman_element *element)
 {
 	if (element->type == CONNMAN_ELEMENT_TYPE_NETWORK &&
 						element->network != NULL)
@@ -293,7 +327,188 @@ const char *__connman_element_get_network(struct connman_element *element)
 	if (element->parent == NULL)
 		return NULL;
 
-	return __connman_element_get_network(element->parent);
+	return __connman_element_get_network_path(element->parent);
+}
+
+struct find_data {
+	enum connman_device_type type;
+	struct connman_device *device;
+};
+
+static gboolean find_device(GNode *node, gpointer user_data)
+{
+	struct connman_element *element = node->data;
+	struct find_data *data = user_data;
+
+	if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
+		return FALSE;
+
+	if (element->device == NULL)
+		return FALSE;
+
+	if (data->type != connman_device_get_type(element->device))
+		return FALSE;
+
+	data->device = element->device;
+
+	return TRUE;
+}
+
+struct connman_device *__connman_element_find_device(enum connman_device_type type)
+{
+	struct find_data data = { .type = type, .device = NULL };
+
+	g_node_traverse(element_root, G_PRE_ORDER,
+				G_TRAVERSE_ALL, -1, find_device, &data);
+
+	return data.device;
+}
+
+static gboolean request_scan(GNode *node, gpointer user_data)
+{
+	struct connman_element *element = node->data;
+	struct find_data *data = user_data;
+	enum connman_device_type type;
+
+	if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
+		return FALSE;
+
+	if (element->device == NULL)
+		return FALSE;
+
+	type = connman_device_get_type(element->device);
+
+	switch (type) {
+	case CONNMAN_DEVICE_TYPE_UNKNOWN:
+	case CONNMAN_DEVICE_TYPE_VENDOR:
+	case CONNMAN_DEVICE_TYPE_ETHERNET:
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+	case CONNMAN_DEVICE_TYPE_GPS:
+	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_HSO:
+	case CONNMAN_DEVICE_TYPE_NOZOMI:
+	case CONNMAN_DEVICE_TYPE_HUAWEI:
+	case CONNMAN_DEVICE_TYPE_NOVATEL:
+		return FALSE;
+	case CONNMAN_DEVICE_TYPE_WIFI:
+	case CONNMAN_DEVICE_TYPE_WIMAX:
+		if (data->type != CONNMAN_DEVICE_TYPE_UNKNOWN &&
+							data->type != type)
+			return FALSE;
+		break;
+	}
+
+	__connman_device_scan(element->device);
+
+	return FALSE;
+}
+
+int __connman_element_request_scan(enum connman_device_type type)
+{
+	struct find_data data = { .type = type, .device = NULL };
+
+	g_node_traverse(element_root, G_PRE_ORDER,
+				G_TRAVERSE_ALL, -1, request_scan, &data);
+
+	return 0;
+}
+
+static gboolean enable_technology(GNode *node, gpointer user_data)
+{
+	struct connman_element *element = node->data;
+	struct find_data *data = user_data;
+	enum connman_device_type type;
+
+	if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
+		return FALSE;
+
+	if (element->device == NULL)
+		return FALSE;
+
+	type = connman_device_get_type(element->device);
+
+	switch (type) {
+	case CONNMAN_DEVICE_TYPE_UNKNOWN:
+	case CONNMAN_DEVICE_TYPE_VENDOR:
+	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_HSO:
+	case CONNMAN_DEVICE_TYPE_NOZOMI:
+	case CONNMAN_DEVICE_TYPE_HUAWEI:
+	case CONNMAN_DEVICE_TYPE_NOVATEL:
+		return FALSE;
+	case CONNMAN_DEVICE_TYPE_ETHERNET:
+	case CONNMAN_DEVICE_TYPE_WIFI:
+	case CONNMAN_DEVICE_TYPE_WIMAX:
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+	case CONNMAN_DEVICE_TYPE_GPS:
+		if (data->type != CONNMAN_DEVICE_TYPE_UNKNOWN &&
+							data->type != type)
+			return FALSE;
+		break;
+	}
+
+	__connman_device_enable(element->device);
+
+	return FALSE;
+}
+
+int __connman_element_enable_technology(enum connman_device_type type)
+{
+	struct find_data data = { .type = type, .device = NULL };
+
+	g_node_traverse(element_root, G_PRE_ORDER,
+				G_TRAVERSE_ALL, -1, enable_technology, &data);
+
+	return 0;
+}
+
+static gboolean disable_technology(GNode *node, gpointer user_data)
+{
+	struct connman_element *element = node->data;
+	struct find_data *data = user_data;
+	enum connman_device_type type;
+
+	if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
+		return FALSE;
+
+	if (element->device == NULL)
+		return FALSE;
+
+	type = connman_device_get_type(element->device);
+
+	switch (type) {
+	case CONNMAN_DEVICE_TYPE_UNKNOWN:
+	case CONNMAN_DEVICE_TYPE_VENDOR:
+	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_HSO:
+	case CONNMAN_DEVICE_TYPE_NOZOMI:
+	case CONNMAN_DEVICE_TYPE_HUAWEI:
+	case CONNMAN_DEVICE_TYPE_NOVATEL:
+		return FALSE;
+	case CONNMAN_DEVICE_TYPE_ETHERNET:
+	case CONNMAN_DEVICE_TYPE_WIFI:
+	case CONNMAN_DEVICE_TYPE_WIMAX:
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+	case CONNMAN_DEVICE_TYPE_GPS:
+		if (data->type != CONNMAN_DEVICE_TYPE_UNKNOWN &&
+							data->type != type)
+			return FALSE;
+		break;
+	}
+
+	__connman_device_disable(element->device);
+
+	return FALSE;
+}
+
+int __connman_element_disable_technology(enum connman_device_type type)
+{
+	struct find_data data = { .type = type, .device = NULL };
+
+	g_node_traverse(element_root, G_PRE_ORDER,
+				G_TRAVERSE_ALL, -1, disable_technology, &data);
+
+	return 0;
 }
 
 static gint compare_priority(gconstpointer a, gconstpointer b)
@@ -433,6 +648,8 @@ void __connman_element_initialize(struct connman_element *element)
 
 	element->name    = NULL;
 	element->type    = CONNMAN_ELEMENT_TYPE_UNKNOWN;
+	element->state   = CONNMAN_ELEMENT_STATE_UNKNOWN;
+	element->error   = CONNMAN_ELEMENT_ERROR_UNKNOWN;
 	element->index   = -1;
 	element->enabled = FALSE;
 
@@ -510,7 +727,7 @@ void connman_element_unref(struct connman_element *element)
 	}
 }
 
-int connman_element_set_static_property(struct connman_element *element,
+static int set_static_property(struct connman_element *element,
 				const char *name, int type, const void *value)
 {
 	struct connman_property *property;
@@ -549,7 +766,7 @@ int connman_element_set_static_property(struct connman_element *element,
 	return 0;
 }
 
-int connman_element_set_static_array_property(struct connman_element *element,
+static int set_static_array_property(struct connman_element *element,
 			const char *name, int type, const void *value, int len)
 {
 	struct connman_property *property;
@@ -589,7 +806,8 @@ int connman_element_set_static_array_property(struct connman_element *element,
 	return 0;
 }
 
-int connman_element_set_property(struct connman_element *element,
+#if 0
+static int set_property(struct connman_element *element,
 				enum connman_property_id id, const void *value)
 {
 	switch (id) {
@@ -629,6 +847,7 @@ int connman_element_set_property(struct connman_element *element,
 
 	return 0;
 }
+#endif
 
 int connman_element_get_value(struct connman_element *element,
 				enum connman_property_id id, void *value)
@@ -638,11 +857,11 @@ int connman_element_get_value(struct connman_element *element,
 
 	switch (id) {
 	case CONNMAN_PROPERTY_ID_IPV4_METHOD:
-		if (element->ipv4.method == CONNMAN_IPV4_METHOD_UNKNOWN)
+		if (element->ipv4.method == CONNMAN_IPCONFIG_METHOD_UNKNOWN)
 			return connman_element_get_value(element->parent,
 								id, value);
 		__connman_element_lock(element);
-		*((const char **) value) = __connman_ipv4_method2string(element->ipv4.method);
+		*((const char **) value) = __connman_ipconfig_method2string(element->ipv4.method);
 		__connman_element_unlock(element);
 		break;
 	case CONNMAN_PROPERTY_ID_IPV4_ADDRESS:
@@ -692,7 +911,7 @@ int connman_element_get_value(struct connman_element *element,
 	return 0;
 }
 
-gboolean connman_element_get_static_property(struct connman_element *element,
+static gboolean get_static_property(struct connman_element *element,
 						const char *name, void *value)
 {
 	struct connman_property *property;
@@ -719,14 +938,13 @@ gboolean connman_element_get_static_property(struct connman_element *element,
 	__connman_element_unlock(element);
 
 	if (found == FALSE && element->parent != NULL)
-		return connman_element_get_static_property(element->parent,
-								name, value);
+		return get_static_property(element->parent, name, value);
 
 	return found;
 }
 
-gboolean connman_element_get_static_array_property(struct connman_element *element,
-					const char *name, void *value, int *len)
+static gboolean get_static_array_property(struct connman_element *element,
+			const char *name, void *value, unsigned int *len)
 {
 	struct connman_property *property;
 	gboolean found = FALSE;
@@ -737,7 +955,7 @@ gboolean connman_element_get_static_array_property(struct connman_element *eleme
 
 	property = g_hash_table_lookup(element->properties, name);
 	if (property != NULL) {
-		*((char **) value) = property->value;
+		*((void **) value) = property->value;
 		*len = property->size;
 		found = TRUE;
 	}
@@ -747,7 +965,8 @@ gboolean connman_element_get_static_array_property(struct connman_element *eleme
 	return found;
 }
 
-gboolean connman_element_match_static_property(struct connman_element *element,
+#if 0
+static gboolean match_static_property(struct connman_element *element,
 					const char *name, const void *value)
 {
 	struct connman_property *property;
@@ -768,12 +987,113 @@ gboolean connman_element_match_static_property(struct connman_element *element,
 
 	return result;
 }
+#endif
+
+/**
+ * connman_element_set_string:
+ * @element: element structure
+ * @key: unique identifier
+ * @value: string value
+ *
+ * Set string value for specific key
+ */
+int connman_element_set_string(struct connman_element *element,
+					const char *key, const char *value)
+{
+	return set_static_property(element, key, DBUS_TYPE_STRING, &value);
+}
+
+/**
+ * connman_element_get_string:
+ * @element: element structure
+ * @key: unique identifier
+ *
+ * Get string value for specific key
+ */
+const char *connman_element_get_string(struct connman_element *element,
+							const char *key)
+{
+	const char *value;
+
+	if (get_static_property(element, key, &value) == FALSE)
+		return NULL;
+
+	return value;
+}
+
+/**
+ * connman_element_set_uint8:
+ * @element: element structure
+ * @key: unique identifier
+ * @value: integer value
+ *
+ * Set integer value for specific key
+ */
+int connman_element_set_uint8(struct connman_element *element,
+					const char *key, connman_uint8_t value)
+{
+        return set_static_property(element, key, DBUS_TYPE_BYTE, &value);
+}
+
+/**
+ * connman_element_get_uint8:
+ * @element: element structure
+ * @key: unique identifier
+ *
+ * Get integer value for specific key
+ */
+connman_uint8_t connman_element_get_uint8(struct connman_element *element,
+							const char *key)
+{
+	connman_uint8_t value;
+
+	if (get_static_property(element, key, &value) == FALSE)
+                return 0;
+
+	return value;
+}
+
+/**
+ * connman_element_set_blob:
+ * @element: element structure
+ * @key: unique identifier
+ * @data: blob data
+ * @size: blob size
+ *
+ * Set binary blob value for specific key
+ */
+int connman_element_set_blob(struct connman_element *element,
+			const char *key, const void *data, unsigned int size)
+{
+	return set_static_array_property(element, key,
+						DBUS_TYPE_BYTE, &data, size);
+}
+
+/**
+ * connman_element_get_blob:
+ * @element: element structure
+ * @key: unique identifier
+ * @size: pointer to blob size
+ *
+ * Get binary blob value for specific key
+ */
+const void *connman_element_get_blob(struct connman_element *element,
+					const char *key, unsigned int *size)
+{
+	void *value;
+
+	if (get_static_array_property(element, key, &value, size) == FALSE)
+		return NULL;
+
+	return value;
+}
 
 int __connman_element_append_ipv4(struct connman_element *element,
 						DBusMessageIter *dict)
 {
 	const char *method = NULL;
 	const char *address = NULL, *netmask = NULL, *gateway = NULL;
+	const char *broadcast = NULL, *nameserver = NULL;
 
 	connman_element_get_value(element,
 				CONNMAN_PROPERTY_ID_IPV4_METHOD, &method);
@@ -784,6 +1104,10 @@ int __connman_element_append_ipv4(struct connman_element *element,
 				CONNMAN_PROPERTY_ID_IPV4_NETMASK, &netmask);
 	connman_element_get_value(element,
 				CONNMAN_PROPERTY_ID_IPV4_GATEWAY, &gateway);
+	connman_element_get_value(element,
+			CONNMAN_PROPERTY_ID_IPV4_BROADCAST, &broadcast);
+	connman_element_get_value(element,
+			CONNMAN_PROPERTY_ID_IPV4_NAMESERVER, &nameserver);
 
 	if (method != NULL)
 		connman_dbus_dict_append_variant(dict, "IPv4.Method",
@@ -801,6 +1125,14 @@ int __connman_element_append_ipv4(struct connman_element *element,
 		connman_dbus_dict_append_variant(dict, "IPv4.Gateway",
 						DBUS_TYPE_STRING, &gateway);
 
+	if (broadcast != NULL)
+		connman_dbus_dict_append_variant(dict, "IPv4.Broadcast",
+						DBUS_TYPE_STRING, &broadcast);
+
+	if (nameserver != NULL)
+		connman_dbus_dict_append_variant(dict, "IPv4.Nameserver",
+						DBUS_TYPE_STRING, &nameserver);
+
 	return 0;
 }
 
@@ -812,15 +1144,15 @@ int __connman_element_set_ipv4(struct connman_element *element,
 	type = dbus_message_iter_get_arg_type(value);
 
 	if (g_str_equal(name, "IPv4.Method") == TRUE) {
-		enum connman_ipv4_method method;
+		enum connman_ipconfig_method method;
 		const char *str;
 
 		if (type != DBUS_TYPE_STRING)
 			return -EINVAL;
 
 		dbus_message_iter_get_basic(value, &str);
-		method = __connman_ipv4_string2method(str);
-		if (method == CONNMAN_IPV4_METHOD_UNKNOWN)
+		method = __connman_ipconfig_string2method(str);
+		if (method == CONNMAN_IPCONFIG_METHOD_UNKNOWN)
 			return -EINVAL;
 
 		if (method == element->ipv4.method)
@@ -864,50 +1196,36 @@ int __connman_element_set_ipv4(struct connman_element *element,
 		g_free(element->ipv4.gateway);
 		element->ipv4.gateway = g_strdup(gateway);
 
+		connman_element_update(element);
+	} else if (g_str_equal(name, "IPv4.Broadcast") == TRUE) {
+		const char *broadcast;
+
+		if (type != DBUS_TYPE_STRING)
+			return -EINVAL;
+
+		dbus_message_iter_get_basic(value, &broadcast);
+
+		g_free(element->ipv4.broadcast);
+		element->ipv4.broadcast = g_strdup(broadcast);
+
+		connman_element_update(element);
+	} else if (g_str_equal(name, "IPv4.Nameserver") == TRUE) {
+		const char *nameserver;
+
+		if (type != DBUS_TYPE_STRING)
+			return -EINVAL;
+
+		dbus_message_iter_get_basic(value, &nameserver);
+
+		g_free(element->ipv4.nameserver);
+		element->ipv4.nameserver = g_strdup(nameserver);
+
 		connman_element_update(element);
 	}
 
 	return 0;
 }
 
-static void append_connections(DBusMessageIter *entry)
-{
-	DBusMessageIter value, iter;
-	const char *key = "Connections";
-
-	dbus_message_iter_append_basic(entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
-		DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_OBJECT_PATH_AS_STRING,
-								&value);
-
-	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
-				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
-	__connman_element_list(NULL, CONNMAN_ELEMENT_TYPE_CONNECTION, &iter);
-	dbus_message_iter_close_container(&value, &iter);
-
-	dbus_message_iter_close_container(entry, &value);
-}
-
-static void emit_connections_signal(DBusConnection *conn)
-{
-	DBusMessage *signal;
-	DBusMessageIter entry;
-
-	DBG("conn %p", conn);
-
-	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
-				CONNMAN_MANAGER_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	append_connections(&entry);
-
-	g_dbus_send_message(conn, signal);
-}
-
 static void append_state(DBusMessageIter *entry, const char *state)
 {
 	DBusMessageIter value;
@@ -938,6 +1256,16 @@ static void emit_state_change(DBusConnection *conn, const char *state)
 	append_state(&entry, state);
 
 	g_dbus_send_message(conn, signal);
+
+	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
+				CONNMAN_MANAGER_INTERFACE, "StateChanged");
+	if (signal == NULL)
+		return;
+
+	dbus_message_iter_init_append(signal, &entry);
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &state);
+
+	g_dbus_send_message(conn, signal);
 }
 
 static void probe_element(struct connman_element *element)
@@ -979,13 +1307,19 @@ static void register_element(gpointer data, gpointer user_data)
 		element->parent = element_root->data;
 
 		node = element_root;
-		basepath = "";
+		basepath = "/device";
 	}
 
 	element->path = g_strdup_printf("%s/%s", basepath, element->name);
 
 	__connman_element_unlock(element);
 
+	if (node == NULL) {
+		connman_error("Element registration for %s failed",
+							element->path);
+		return;
+	}
+
 	DBG("element %p path %s", element, element->path);
 
 	g_node_append_data(node, element);
@@ -1006,8 +1340,6 @@ static void register_element(gpointer data, gpointer user_data)
 			parent = parent->parent;
 		}
 
-		emit_connections_signal(connection);
-
 		if (__connman_element_count(NULL,
 					CONNMAN_ELEMENT_TYPE_CONNECTION) == 1)
 			emit_state_change(connection, "online");
@@ -1062,7 +1394,7 @@ int connman_element_register(struct connman_element *element,
 	}
 
 	if (element->type == CONNMAN_ELEMENT_TYPE_DHCP)
-		element->ipv4.method = CONNMAN_IPV4_METHOD_DHCP;
+		element->ipv4.method = CONNMAN_IPCONFIG_METHOD_DHCP;
 
 	element->parent = parent;
 
@@ -1102,8 +1434,6 @@ static gboolean remove_element(GNode *node, gpointer user_data)
 		if (__connman_element_count(NULL,
 					CONNMAN_ELEMENT_TYPE_CONNECTION) == 0)
 			emit_state_change(connection, "offline");
-
-		emit_connections_signal(connection);
 	}
 
 	emit_element_signal(connection, "ElementRemoved", element);
@@ -1174,9 +1504,53 @@ int connman_element_set_enabled(struct connman_element *element,
 
 	element->enabled = enabled;
 
+	connman_element_update(element);
+
 	return 0;
 }
 
+static enum connman_service_error convert_error(enum connman_element_error error)
+{
+	switch (error) {
+	case CONNMAN_ELEMENT_ERROR_UNKNOWN:
+	case CONNMAN_ELEMENT_ERROR_FAILED:
+		break;
+	case CONNMAN_ELEMENT_ERROR_DHCP_FAILED:
+		return CONNMAN_SERVICE_ERROR_DHCP_FAILED;
+	case CONNMAN_ELEMENT_ERROR_CONNECT_FAILED:
+		return CONNMAN_SERVICE_ERROR_CONNECT_FAILED;
+	}
+
+	return CONNMAN_SERVICE_ERROR_UNKNOWN;
+}
+
+/**
+ * connman_element_set_error:
+ * @element: element structure
+ * @error: error identifier
+ *
+ * Set error state and specific error identifier
+ */
+void connman_element_set_error(struct connman_element *element,
+					enum connman_element_error error)
+{
+	struct connman_service *service;
+
+	DBG("element %p error %d", element, error);
+
+	if (element->type == CONNMAN_ELEMENT_TYPE_ROOT)
+		return;
+
+	element->state = CONNMAN_ELEMENT_STATE_ERROR;
+	element->error = error;
+
+	if (element->driver && element->driver->change)
+		element->driver->change(element);
+
+	service = __connman_element_get_service(element);
+	__connman_service_indicate_error(service, convert_error(error));
+}
+
 int __connman_element_init(DBusConnection *conn, const char *device,
 							const char *nodevice)
 {
@@ -1197,6 +1571,8 @@ int __connman_element_init(DBusConnection *conn, const char *device,
 
 	element_root = g_node_new(element);
 
+	__connman_notifier_init();
+	__connman_service_init();
 	__connman_network_init();
 	__connman_device_init();
 
@@ -1233,14 +1609,14 @@ void __connman_element_start(void)
 
 	__connman_connection_init();
 	__connman_ipv4_init();
-	__connman_detect_init();
+	__connman_rfkill_init();
 }
 
 void __connman_element_stop(void)
 {
 	DBG("");
 
-	__connman_detect_cleanup();
+	__connman_rfkill_cleanup();
 	__connman_ipv4_cleanup();
 	__connman_connection_cleanup();
 }
@@ -1281,6 +1657,8 @@ void __connman_element_cleanup(void)
 
 	__connman_device_cleanup();
 	__connman_network_cleanup();
+	__connman_service_cleanup();
+	__connman_notifier_cleanup();
 
 	g_node_traverse(element_root, G_POST_ORDER, G_TRAVERSE_ALL, -1,
 							free_driver, NULL);
