@@ -187,10 +187,14 @@ static const char *error2string(enum connman_service_error error)
 	switch (error) {
 	case CONNMAN_SERVICE_ERROR_UNKNOWN:
 		break;
-	case CONNMAN_SERVICE_ERROR_DHCP_FAILED:
-		return "dhcp-failed";
+	case CONNMAN_SERVICE_ERROR_OUT_OF_RANGE:
+		return "out-of-range";
 	case CONNMAN_SERVICE_ERROR_PIN_MISSING:
 		return "pin-missing";
+	case CONNMAN_SERVICE_ERROR_DHCP_FAILED:
+		return "dhcp-failed";
+	case CONNMAN_SERVICE_ERROR_CONNECT_FAILED:
+		return "connect-failed";
 	}
 
 	return NULL;
@@ -392,6 +396,13 @@ static DBusMessage *set_property(DBusConnection *conn,
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
+static void set_idle(struct connman_service *service)
+{
+	service->state = CONNMAN_SERVICE_STATE_IDLE;
+	service->error = CONNMAN_SERVICE_ERROR_UNKNOWN;
+	state_changed(service);
+}
+
 static DBusMessage *clear_property(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
@@ -408,9 +419,7 @@ static DBusMessage *clear_property(DBusConnection *conn,
 		return __connman_error_permission_denied(msg);
 
 	if (g_str_equal(name, "Error") == TRUE) {
-		service->state = CONNMAN_SERVICE_STATE_IDLE;
-		service->error = CONNMAN_SERVICE_ERROR_UNKNOWN;
-		state_changed(service);
+		set_idle(service);
 
 		g_get_current_time(&service->modified);
 		__connman_storage_save_service(service);
@@ -438,24 +447,6 @@ static connman_bool_t is_connecting(struct connman_service *service)
 	return FALSE;
 }
 
-static struct connman_service *find_pending_service(void)
-{
-	struct connman_service *service;
-	GSequenceIter *iter;
-
-	iter = g_sequence_get_begin_iter(service_list);
-
-	while (g_sequence_iter_is_end(iter) == FALSE) {
-		service = g_sequence_get(iter);
-		if (service->pending != NULL)
-			return service;
-
-		iter = g_sequence_iter_next(iter);
-	}
-
-	return NULL;
-}
-
 static connman_bool_t is_ignore(struct connman_service *service)
 {
 	if (service->ignore == TRUE)
@@ -467,43 +458,67 @@ static connman_bool_t is_ignore(struct connman_service *service)
 	return FALSE;
 }
 
-static void __connman_service_auto_connect(void)
+void __connman_service_auto_connect(void)
 {
-	struct connman_service *service;
+	struct connman_service *service = NULL;
 	GSequenceIter *iter;
 
 	DBG("");
 
-	service = find_pending_service();
-	if (service != NULL)
-		return;
-
 	iter = g_sequence_get_begin_iter(service_list);
-	if (g_sequence_iter_is_end(iter) == TRUE)
-		return;
 
-	service = g_sequence_get(iter);
-
-	while (is_ignore(service) == TRUE) {
-		iter = g_sequence_iter_next(iter);
-		if (g_sequence_iter_is_end(iter))
-			return;
+	while (g_sequence_iter_is_end(iter) == FALSE) {
 		service = g_sequence_get(iter);
+
+		if (service->pending != NULL)
+			return;
+
+		if (is_connecting(service) == TRUE)
+			return;
+
+		if (service->favorite == FALSE)
+			return;
+
+		if (service->state == CONNMAN_SERVICE_STATE_READY)
+			return;
+
+		if (is_ignore(service) == FALSE &&
+				service->state == CONNMAN_SERVICE_STATE_IDLE)
+			break;
+
+		service = NULL;
+
+		iter = g_sequence_iter_next(iter);
 	}
 
-	if (service->favorite == FALSE)
-		return;
-
-	if (service->state == CONNMAN_SERVICE_STATE_READY)
-		return;
-
-	if (is_connecting(service) == TRUE)
-		return;
-
-	if (service->state == CONNMAN_SERVICE_STATE_IDLE)
+	if (service != NULL)
 		__connman_service_connect(service);
 }
 
+static void reply_pending(struct connman_service *service, int error)
+{
+	if (service->timeout > 0) {
+		g_source_remove(service->timeout);
+		service->timeout = 0;
+	}
+
+	if (service->pending != NULL) {
+		if (error > 0) {
+			DBusMessage *reply;
+
+			reply = __connman_error_failed(service->pending,
+								error);
+			if (reply != NULL)
+				g_dbus_send_message(connection, reply);
+		} else
+			g_dbus_send_reply(connection, service->pending,
+							DBUS_TYPE_INVALID);
+
+		dbus_message_unref(service->pending);
+		service->pending = NULL;
+	}
+}
+
 static gboolean connect_timeout(gpointer user_data)
 {
 	struct connman_service *service = user_data;
@@ -562,6 +577,7 @@ static DBusMessage *connect_service(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
 	struct connman_service *service = user_data;
+	GSequenceIter *iter;
 	int err;
 
 	DBG("service %p", service);
@@ -569,6 +585,18 @@ static DBusMessage *connect_service(DBusConnection *conn,
 	if (service->pending != NULL)
 		return __connman_error_in_progress(msg);
 
+	iter = g_sequence_get_begin_iter(service_list);
+
+	while (g_sequence_iter_is_end(iter) == FALSE) {
+		struct connman_service *temp = g_sequence_get(iter);
+
+		if (service->type == temp->type &&
+					is_connecting(temp) == TRUE)
+			return __connman_error_in_progress(msg);
+
+		iter = g_sequence_iter_next(iter);
+	}
+
 	service->ignore = FALSE;
 
 	service->pending = dbus_message_ref(msg);
@@ -599,19 +627,6 @@ static DBusMessage *disconnect_service(DBusConnection *conn,
 
 	DBG("service %p", service);
 
-	if (service->pending != NULL) {
-		DBusMessage *reply;
-
-		reply = __connman_error_operation_aborted(service->pending);
-		if (reply != NULL)
-			g_dbus_send_message(conn, reply);
-
-		dbus_message_unref(service->pending);
-		service->pending = NULL;
-
-		return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-	}
-
 	service->ignore = TRUE;
 
 	err = __connman_service_disconnect(service);
@@ -680,7 +695,7 @@ static DBusMessage *move_before(DBusConnection *conn,
 
 	g_sequence_move(src, dst);
 
-	__connman_profile_changed();
+	__connman_profile_changed(FALSE);
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
@@ -737,22 +752,14 @@ static void service_free(gpointer user_data)
 
 	DBG("service %p", service);
 
+	reply_pending(service, ENOENT);
+
 	g_hash_table_remove(service_hash, service->identifier);
 
-	if (service->timeout > 0) {
-		g_source_remove(service->timeout);
-		service->timeout = 0;
-	}
-
-	if (service->pending != NULL) {
-		dbus_message_unref(service->pending);
-		service->pending = NULL;
-	}
-
 	service->path = NULL;
 
 	if (path != NULL) {
-		__connman_profile_changed();
+		__connman_profile_changed(FALSE);
 
 		g_dbus_unregister_interface(connection, path,
 						CONNMAN_SERVICE_INTERFACE);
@@ -785,9 +792,10 @@ void __connman_service_put(struct connman_service *service)
 		GSequenceIter *iter;
 
 		iter = g_hash_table_lookup(service_hash, service->identifier);
-		if (iter != NULL)
+		if (iter != NULL) {
+			__connman_service_disconnect(service);
 			g_sequence_remove(iter);
-		else
+		} else
 			service_free(service);
 	}
 }
@@ -917,7 +925,7 @@ int connman_service_set_favorite(struct connman_service *service,
 
 	g_sequence_sort_changed(iter, service_compare, NULL);
 
-	__connman_profile_changed();
+	__connman_profile_changed(FALSE);
 
 	return 0;
 }
@@ -971,6 +979,10 @@ int __connman_service_indicate_state(struct connman_service *service,
 	if (service->state == state)
 		return -EALREADY;
 
+	if (service->state == CONNMAN_SERVICE_STATE_FAILURE &&
+				state == CONNMAN_SERVICE_STATE_IDLE)
+		return -EINVAL;
+
 	if (service->state == CONNMAN_SERVICE_STATE_IDLE &&
 				state == CONNMAN_SERVICE_STATE_DISCONNECT)
 		return -EINVAL;
@@ -979,6 +991,8 @@ int __connman_service_indicate_state(struct connman_service *service,
 			service->state != CONNMAN_SERVICE_STATE_DISCONNECT) {
 		service->state = CONNMAN_SERVICE_STATE_DISCONNECT;
 		state_changed(service);
+
+		__connman_service_disconnect(service);
 	}
 
 	service->state = state;
@@ -987,39 +1001,14 @@ int __connman_service_indicate_state(struct connman_service *service,
 	if (state == CONNMAN_SERVICE_STATE_READY) {
 		connman_service_set_favorite(service, TRUE);
 
-		if (service->timeout > 0) {
-			g_source_remove(service->timeout);
-			service->timeout = 0;
-		}
-
-		if (service->pending != NULL) {
-			g_dbus_send_reply(connection, service->pending,
-							DBUS_TYPE_INVALID);
-
-			dbus_message_unref(service->pending);
-			service->pending = NULL;
-		}
+		reply_pending(service, 0);
 
 		g_get_current_time(&service->modified);
 		__connman_storage_save_service(service);
 	}
 
 	if (state == CONNMAN_SERVICE_STATE_FAILURE) {
-		if (service->timeout > 0) {
-			g_source_remove(service->timeout);
-			service->timeout = 0;
-		}
-
-		if (service->pending != NULL) {
-			DBusMessage *reply;
-
-			reply = __connman_error_failed(service->pending, EIO);
-			if (reply != NULL)
-				g_dbus_send_message(connection, reply);
-
-			dbus_message_unref(service->pending);
-			service->pending = NULL;
-		}
+		reply_pending(service, EIO);
 
 		g_get_current_time(&service->modified);
 		__connman_storage_save_service(service);
@@ -1030,7 +1019,11 @@ int __connman_service_indicate_state(struct connman_service *service,
 	if (iter != NULL)
 		g_sequence_sort_changed(iter, service_compare, NULL);
 
-	__connman_profile_changed();
+	__connman_profile_changed(FALSE);
+
+	if (service->state == CONNMAN_SERVICE_STATE_IDLE ||
+			service->state == CONNMAN_SERVICE_STATE_FAILURE)
+		__connman_element_request_scan(CONNMAN_ELEMENT_TYPE_UNKNOWN);
 
 	return 0;
 }
@@ -1065,12 +1058,16 @@ int __connman_service_connect(struct connman_service *service)
 	if (service->state == CONNMAN_SERVICE_STATE_READY)
 		return -EISCONN;
 
+	if (is_connecting(service) == TRUE)
+		return -EALREADY;
+
 	if (service->network != NULL) {
 		unsigned int ssid_len;
 
 		if (connman_network_get_blob(service->network, "WiFi.SSID",
-					     &ssid_len) == NULL)
+						     &ssid_len) == NULL)
 			return -EINVAL;
+
 		connman_network_set_string(service->network,
 				"WiFi.Passphrase", service->passphrase);
 
@@ -1102,6 +1099,8 @@ int __connman_service_disconnect(struct connman_service *service)
 
 	DBG("service %p", service);
 
+	reply_pending(service, ECONNABORTED);
+
 	if (service->network != NULL) {
 		err = __connman_network_disconnect(service->network);
 	} else if (service->device != NULL) {
@@ -1138,6 +1137,190 @@ static struct connman_service *__connman_service_lookup(const char *identifier)
 	return NULL;
 }
 
+static struct connman_network *create_hidden_wifi(struct connman_device *device,
+		const char *ssid, const char *mode, const char *security)
+{
+	struct connman_network *network;
+	char *name;
+	int index;
+	unsigned int i, ssid_len;
+
+	ssid_len = strlen(ssid);
+	if (ssid_len < 1)
+		return NULL;
+
+	network = connman_network_create(NULL, CONNMAN_NETWORK_TYPE_WIFI);
+	if (network == NULL)
+		return NULL;
+
+	connman_network_set_blob(network, "WiFi.SSID",
+					(unsigned char *) ssid, ssid_len);
+
+	connman_network_set_string(network, "WiFi.Mode", mode);
+	connman_network_set_string(network, "WiFi.Security", security);
+
+	name = g_try_malloc0(ssid_len + 1);
+	if (name == NULL) {
+		connman_network_unref(network);
+		return NULL;
+	}
+
+	for (i = 0; i < ssid_len; i++) {
+		if (g_ascii_isprint(ssid[i]))
+			name[i] = ssid[i];
+		else
+			name[i] = ' ';
+	}
+
+	connman_network_set_name(network, name);
+
+	g_free(name);
+
+	index = connman_device_get_index(device);
+	connman_network_set_index(network, index);
+
+	connman_network_set_protocol(network, CONNMAN_NETWORK_PROTOCOL_IP);
+
+	if (connman_device_add_network(device, network) < 0) {
+		connman_network_unref(network);
+		return NULL;
+	}
+
+	connman_network_set_available(network, TRUE);
+
+	return network;
+}
+
+int __connman_service_create_and_connect(DBusMessage *msg)
+{
+	struct connman_service *service;
+	struct connman_network *network;
+	struct connman_device *device;
+	DBusMessageIter iter, array;
+	const char *mode = "managed", *security = "none";
+	const char *type = NULL, *ssid = NULL, *passphrase = NULL;
+	unsigned int ssid_len = 0;
+	const char *ident;
+	char *name, *group;
+	gboolean created = FALSE;
+	int err;
+
+	dbus_message_iter_init(msg, &iter);
+	dbus_message_iter_recurse(&iter, &array);
+
+	while (dbus_message_iter_get_arg_type(&array) == DBUS_TYPE_DICT_ENTRY) {
+		DBusMessageIter entry, value;
+		const char *key;
+
+		dbus_message_iter_recurse(&array, &entry);
+		dbus_message_iter_get_basic(&entry, &key);
+
+		dbus_message_iter_next(&entry);
+		dbus_message_iter_recurse(&entry, &value);
+
+		switch (dbus_message_iter_get_arg_type(&value)) {
+		case DBUS_TYPE_STRING:
+			if (g_str_equal(key, "Type") == TRUE)
+				dbus_message_iter_get_basic(&value, &type);
+			else if (g_str_equal(key, "WiFi.Mode") == TRUE ||
+					g_str_equal(key, "Mode") == TRUE)
+				dbus_message_iter_get_basic(&value, &mode);
+			else if (g_str_equal(key, "WiFi.Security") == TRUE ||
+					g_str_equal(key, "Security") == TRUE)
+				dbus_message_iter_get_basic(&value, &security);
+			else if (g_str_equal(key, "WiFi.Passphrase") == TRUE ||
+					g_str_equal(key, "Passphrase") == TRUE)
+				dbus_message_iter_get_basic(&value, &passphrase);
+			else if (g_str_equal(key, "WiFi.SSID") == TRUE ||
+					g_str_equal(key, "SSID") == TRUE)
+				dbus_message_iter_get_basic(&value, &ssid);
+		}
+
+		dbus_message_iter_next(&array);
+	}
+
+	if (type == NULL)
+		return -EINVAL;
+
+	if (g_strcmp0(type, "wifi") != 0 || g_strcmp0(mode, "managed") != 0)
+		return -EOPNOTSUPP;
+
+	if (ssid == NULL)
+		return -EINVAL;
+
+	ssid_len = strlen(ssid);
+	if (ssid_len < 1)
+		return -EINVAL;
+
+	device = __connman_element_find_device(CONNMAN_DEVICE_TYPE_WIFI);
+	if (device == NULL)
+		return -EOPNOTSUPP;
+
+	ident = __connman_device_get_ident(device);
+	if (ident == NULL)
+		return -EOPNOTSUPP;
+
+	group = connman_wifi_build_group_name((unsigned char *) ssid,
+						ssid_len, mode, security);
+	if (group == NULL)
+		return -EINVAL;
+
+	name = g_strdup_printf("%s_%s_%s", type, ident, group);
+
+	service = __connman_service_lookup(name);
+
+	if (service != NULL)
+		goto done;
+
+	network = create_hidden_wifi(device, ssid, mode, security);
+	if (network != NULL) {
+		connman_network_set_group(network, group);
+		created = TRUE;
+	}
+
+	service = __connman_service_lookup(name);
+
+done:
+	g_free(name);
+	g_free(group);
+
+	if (service == NULL) {
+		err = -EOPNOTSUPP;
+		goto failed;
+	}
+
+	__connman_device_disconnect(device);
+
+	if (passphrase != NULL) {
+		g_free(service->passphrase);
+		service->passphrase = g_strdup(passphrase);
+	}
+
+	err = __connman_service_connect(service);
+	if (err < 0 && err != -EINPROGRESS)
+		goto failed;
+
+	g_dbus_send_reply(connection, msg,
+				DBUS_TYPE_OBJECT_PATH, &service->path,
+							DBUS_TYPE_INVALID);
+
+	return 0;
+
+failed:
+	if (service != NULL && created == TRUE) {
+		struct connman_network *network = service->network;
+
+		if (network != NULL) {
+			connman_network_set_available(network, FALSE);
+			__connman_device_cleanup_networks(device);
+		}
+
+		__connman_service_put(service);
+	}
+
+	return err;
+}
+
 /**
  * __connman_service_get:
  * @identifier: service identifier
@@ -1202,13 +1385,13 @@ static int service_register(struct connman_service *service)
 	if (iter != NULL)
 		g_sequence_sort_changed(iter, service_compare, NULL);
 
-	__connman_profile_changed();
+	__connman_profile_changed(TRUE);
 
 	return 0;
 }
 
 /**
- * connman_service_lookup_from_device:
+ * __connman_service_lookup_from_device:
  * @device: device structure
  *
  * Look up a service by device (reference count will not be increased)
@@ -1225,9 +1408,7 @@ struct connman_service *__connman_service_lookup_from_device(struct connman_devi
 
 	name = g_strdup_printf("%s_%s",
 				__connman_device_get_type(device), ident);
-
 	service = __connman_service_lookup(name);
-
 	g_free(name);
 
 	return service;
@@ -1258,7 +1439,7 @@ static enum connman_service_type convert_device_type(struct connman_device *devi
 }
 
 /**
- * connman_service_create_from_device:
+ * __connman_service_create_from_device:
  * @device: device structure
  *
  * Look up service by device and if not found, create one
@@ -1275,15 +1456,15 @@ struct connman_service *__connman_service_create_from_device(struct connman_devi
 
 	name = g_strdup_printf("%s_%s",
 				__connman_device_get_type(device), ident);
-
 	service = __connman_service_get(name);
+	g_free(name);
+
 	if (service == NULL)
-		goto done;
+		return NULL;
 
 	if (service->path != NULL) {
-		__connman_service_put(service);
-		service = NULL;
-		goto done;
+		__connman_profile_changed(TRUE);
+		return service;
 	}
 
 	service->type = convert_device_type(device);
@@ -1292,17 +1473,27 @@ struct connman_service *__connman_service_create_from_device(struct connman_devi
 
 	service_register(service);
 
+	__connman_profile_changed(TRUE);
+
 	if (service->favorite == TRUE)
 		__connman_service_auto_connect();
 
-done:
-	g_free(name);
-
 	return service;
 }
 
+void __connman_service_remove_from_device(struct connman_device *device)
+{
+	struct connman_service *service;
+
+	service = __connman_service_lookup_from_device(device);
+	if (service == NULL)
+		return;
+
+	__connman_service_put(service);
+}
+
 /**
- * connman_service_lookup_from_network:
+ * __connman_service_lookup_from_network:
  * @network: network structure
  *
  * Look up a service by network (reference count will not be increased)
@@ -1323,9 +1514,7 @@ struct connman_service *__connman_service_lookup_from_network(struct connman_net
 
 	name = g_strdup_printf("%s_%s_%s",
 			__connman_network_get_type(network), ident, group);
-
 	service = __connman_service_lookup(name);
-
 	g_free(name);
 
 	return service;
@@ -1333,9 +1522,25 @@ struct connman_service *__connman_service_lookup_from_network(struct connman_net
 
 unsigned int __connman_service_get_order(struct connman_service *service)
 {
+	GSequenceIter *iter;
+
 	if (service == NULL)
 		return 0;
 
+	if (service->favorite == FALSE) {
+		service->order = 0;
+		goto done;
+	}
+
+	iter = g_hash_table_lookup(service_hash, service->identifier);
+	if (iter != NULL) {
+		if (g_sequence_iter_get_position(iter) == 0)
+			service->order = 1;
+		else
+			service->order = 0;
+	}
+
+done:
 	return service->order;
 }
 
@@ -1348,8 +1553,10 @@ static enum connman_service_type convert_network_type(struct connman_network *ne
 	case CONNMAN_NETWORK_TYPE_VENDOR:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN:
 	case CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN:
-	case CONNMAN_NETWORK_TYPE_HSO:
 		break;
+	case CONNMAN_NETWORK_TYPE_MBM:
+	case CONNMAN_NETWORK_TYPE_HSO:
+		return CONNMAN_SERVICE_TYPE_CELLULAR;
 	case CONNMAN_NETWORK_TYPE_WIFI:
 		return CONNMAN_SERVICE_TYPE_WIFI;
 	case CONNMAN_NETWORK_TYPE_WIMAX:
@@ -1421,28 +1628,21 @@ static void update_from_network(struct connman_service *service,
 
 	if (service->strength > strength && service->network != NULL) {
 		connman_network_unref(service->network);
-		service->network = NULL;
+		service->network = connman_network_ref(network);
 
 		strength_changed(service);
 	}
 
-	if (service->network == NULL) {
+	if (service->network == NULL)
 		service->network = connman_network_ref(network);
 
-		str = connman_network_get_string(network, "WiFi.Passphrase");
-		if (str != NULL) {
-			g_free(service->passphrase);
-			service->passphrase = g_strdup(str);
-		}
-	}
-
 	iter = g_hash_table_lookup(service_hash, service->identifier);
 	if (iter != NULL)
 		g_sequence_sort_changed(iter, service_compare, NULL);
 }
 
 /**
- * connman_service_create_from_network:
+ * __connman_service_create_from_network:
  * @network: network structure
  *
  * Look up service by network and if not found, create one
@@ -1453,12 +1653,6 @@ struct connman_service *__connman_service_create_from_network(struct connman_net
 	const char *ident, *group;
 	char *name;
 
-	service = __connman_service_lookup_from_network(network);
-	if (service != NULL) {
-		update_from_network(service, network);
-		return service;
-	}
-
 	ident = __connman_network_get_ident(network);
 	if (ident == NULL)
 		return NULL;
@@ -1469,19 +1663,19 @@ struct connman_service *__connman_service_create_from_network(struct connman_net
 
 	name = g_strdup_printf("%s_%s_%s",
 			__connman_network_get_type(network), ident, group);
-
 	service = __connman_service_get(name);
+	g_free(name);
+
 	if (service == NULL)
-		goto done;
+		return NULL;
+
+	if (__connman_network_get_weakness(network) == TRUE)
+		return service;
 
 	if (service->path != NULL) {
 		update_from_network(service, network);
-
-		__connman_profile_changed();
-
-		__connman_service_put(service);
-		service = NULL;
-		goto done;
+		__connman_profile_changed(TRUE);
+		return service;
 	}
 
 	service->type = convert_network_type(network);
@@ -1492,15 +1686,25 @@ struct connman_service *__connman_service_create_from_network(struct connman_net
 
 	service_register(service);
 
+	__connman_profile_changed(TRUE);
+
 	if (service->favorite == TRUE)
 		__connman_service_auto_connect();
 
-done:
-	g_free(name);
-
 	return service;
 }
 
+void __connman_service_remove_from_network(struct connman_network *network)
+{
+	struct connman_service *service;
+
+	service = __connman_service_lookup_from_network(network);
+	if (service == NULL)
+		return;
+
+	__connman_service_put(service);
+}
+
 static int service_load(struct connman_service *service)
 {
 	GKeyFile *keyfile;
@@ -1563,7 +1767,7 @@ static int service_load(struct connman_service *service)
 
 			hex_ssid = g_key_file_get_string(keyfile,
 							service->identifier,
-							"WiFi.SSID", NULL);
+								"SSID", NULL);
 
 			if (hex_ssid != NULL) {
 				gchar *ssid;
@@ -1691,7 +1895,7 @@ update:
 							"%02x", ssid[i]);
 
 				g_key_file_set_string(keyfile, identifier,
-							"WiFi.SSID", str->str);
+							"SSID", str->str);
 
 				g_string_free(str, TRUE);
 			}
@@ -1724,7 +1928,7 @@ update:
 		g_free(str);
 	}
 
-	if (service->passphrase != NULL)
+	if (service->passphrase != NULL && strlen(service->passphrase) > 0)
 		g_key_file_set_string(keyfile, service->identifier,
 					"Passphrase", service->passphrase);
 	else
